-- drop tables
--drop index idx_dup;
DROP TABLE duplicate CASCADE CONSTRAINTS ; 

drop trigger fingerprint_autonumber;
drop sequence fingerprint_seq;
--drop index idx_fingerprint;
drop table fingerprint;

--drop index idx_fileinfo;
DROP TABLE fileinfo CASCADE CONSTRAINTS ; 

drop trigger filename_autonumber;
drop sequence filename_seq;
--drop index IDX_filename;
DROP TABLE filename CASCADE CONSTRAINTS ; 

drop trigger fingerprinttype_autonumber;
drop sequence fingerprinttype_seq;
--drop INDEX idx_fingerprinttype;
drop table fingerprinttype;

DROP TRIGGER fileinfotype_autonumber;
DROP SEQUENCE fileinfotype_seq;
DROP TABLE fileinfotype CASCADE CONSTRAINTS ;

DROP TABLE status CASCADE CONSTRAINTS ; 

-- status table
create table status (
  statusid integer not null, 
  status varchar2(50) not null, 
  CONSTRAINT PK_status PRIMARY KEY (statusid), 
  CONSTRAINT UNQ_status UNIQUE (status)
);

-- fileinfotype table
create table fileinfotype (
  fileinfotypeid integer not null, 
  fileinfotype varchar2(500) not null,
  CONSTRAINT PK_fileinfotype PRIMARY KEY (fileinfotypeid), 
  CONSTRAINT UNQ_fileinfotype UNIQUE (fileinfotype)
);
create sequence fileinfotype_seq start with 1 increment by 1;
create or replace trigger fileinfotype_autonumber
before insert on fileinfotype for each row
begin
    if :new.fileinfotypeid is null then
        select fileinfotype_seq.nextval into :new.fileinfotypeid from dual;
    end if;
end;
/

-- fingerprinttype table
create table fingerprinttype (
  fingerprinttypeid integer not null,
  fingerprinttype varchar2(100) not null,
  CONSTRAINT UNQ_fingerprinttype UNIQUE (fingerprinttype),
  CONSTRAINT PK_fingerprinttype PRIMARY KEY ( fingerprinttypeid )
);
--create INDEX idx_fingerprinttype on fingerprinttype(fingerprinttype);
create sequence fingerprinttype_seq start with 1 increment by 1;
create or replace trigger fingerprinttype_autonumber
before insert on fingerprinttype for each row
begin
    if :new.fingerprinttypeid is null then
        select fingerprinttype_seq.nextval into :new.fingerprinttypeid from dual;
    end if;
end;
/

-- filename table
create table filename (
  filenameid integer not null,
  filename varchar2(500) not null,
  statusid integer,
  CONSTRAINT UNQ_filename UNIQUE (filename),
  CONSTRAINT PK_filename PRIMARY KEY ( filenameid )
);
ALTER TABLE filename ADD CONSTRAINT FK_filename_statusid FOREIGN KEY (statusid) REFERENCES status (statusid) ;
--create index IDX_filename on filename(filename);
create sequence filename_seq start with 1 increment by 1;
create or replace trigger filename_autonumber
before insert on filename for each row
begin
    if :new.filenameid is null then
        select filename_seq.nextval into :new.filenameid from dual;
    end if;
end;
/

-- fileinfo table
create table fileinfo (
  fileinfotypeid integer not null,
  filenameid integer not null,
  fileinfo varchar2(500) not null,
  constraint UNQ_fileinfo unique (filenameid, fileinfotypeid)
);
ALTER TABLE fileinfo ADD CONSTRAINT FK_fileinfo_fileinfotypeid FOREIGN KEY (fileinfotypeid) references fileinfotype(fileinfotypeid);
ALTER TABLE fileinfo ADD CONSTRAINT FK_fileinfo_filenameid FOREIGN KEY (filenameid) references filename(filenameid);
--create index idx_fileinfo ON fileinfo(filenameid, fileinfotypeid);

-- fingerprint table
create table fingerprint (
  fingerprintid integer not null,
  filenameid integer not null,
  fingerprinttypeid integer not null,
  fingerprint varchar2(2000),
  constraint UNQ_fingerprint unique (filenameid, fingerprinttypeid),
  CONSTRAINT PK_fingerprint PRIMARY KEY ( fingerprintid )
);
ALTER TABLE fingerprint ADD CONSTRAINT FK_fingerprint_filenameid FOREIGN KEY (filenameid) references filename(filenameid);
ALTER TABLE fingerprint ADD CONSTRAINT FK_fingerprint_fptypeid FOREIGN KEY (fingerprinttypeid) references fingerprinttype(fingerprinttypeid);
--create index idx_fingerprint on fingerprint(filenameid, fingerprinttypeid);
create sequence fingerprint_seq start with 1 increment by 1;
create or replace trigger fingerprint_autonumber
before insert on fingerprint for each row
begin
    if :new.fingerprintid is null then
        select fingerprint_seq.nextval into :new.fingerprintid from dual;
    end if;
end;
/

-- duplicate table
create table duplicate (
  fingerprintid1 integer not null,
  fingerprintid2 integer not null,
  similarity number not null,
  constraint unq_dup unique (fingerprintid1, fingerprintid2)
);
ALTER TABLE duplicate ADD CONSTRAINT FK_duplicate_fingerprintid1 FOREIGN KEY (fingerprintid1) references fingerprint(fingerprintid);
ALTER TABLE duplicate ADD CONSTRAINT FK_duplicate_fingerprintid2 FOREIGN KEY (fingerprintid2) references fingerprint(fingerprintid);
--create index idx_dup on duplicate(fingerprintid1, fingerprintid2);

insert into status values (-1,'unknown');
insert into status values (0,'OK');
insert into status values (1,'too short');
insert into status values (2,'signature mismatch');
insert into status values (3,'not readable');
insert into status values (4,'invalid');
insert into status values (5,'corrupt');
insert into status values (6,'empty');
insert into status values (7,'missing');

commit;
